/*
 * Copyright (c) 2005 Nokia
 * Author: Timo Savola <tsavola@movial.fi>
 *
 * Licensed under GPL, see COPYING for details.
 */

/*
 * The undocumented libelf is similar to its documented Solaris counterpart:
 *   http://docs.sun.com/app/docs/doc/816-5172/6mbb7bt8u?a=expand
 *   http://developers.sun.com/solaris/articles/elf.html
 */

#include <libelf.h>
#include <sys/types.h>
#include <fcntl.h>
#include <iostream>
#include <cstring>
#include <cerrno>

#define SBOX_INTERP "/scratchbox/host_shared/lib/ld.so"

static const char *filename;

static void error(const char *msg)
{
	std::cerr << filename << ": " << msg << std::endl;
	std::exit(1);
}

static void elf_error()
{
	error(elf_errmsg(elf_errno()));
}

int main(int argc, char **argv)
{
	if (argc != 2 || argv[1][0] == '-') {
		std::cerr << "Usage: " << argv[0] << " <filename>" << std::endl;
		return 1;
	}

	filename = argv[1];

	if (elf_version(EV_CURRENT) == EV_NONE)
		elf_error();

	int fd = open(filename, O_RDWR);
	if (fd < 0)
		error(strerror(errno));

	Elf *elf = elf_begin(fd, ELF_C_RDWR, 0);
	if (!elf)
		elf_error();

	if (elf_kind(elf) != ELF_K_ELF)
		error("Not an ELF binary");

	Elf32_Ehdr *ehdr = elf32_getehdr(elf);
	if (!ehdr)
		elf_error();

	unsigned int index = ehdr->e_shstrndx;

	Elf_Scn *scn = 0;
	while ((scn = elf_nextscn(elf, scn))) {
		Elf32_Shdr *shdr = elf32_getshdr(scn);
		if (shdr->sh_type != SHT_PROGBITS)
			continue;

		char *name = elf_strptr(elf, index, shdr->sh_name);
		if (strcmp(name, ".interp") != 0)
			continue;

		Elf_Data *data = elf_getdata(scn, 0);
		if (!data)
			elf_error();

//		Is this necessary?
//		if (data->d_buf)
//			free(data->d_buf);

#if 1
		data->d_size = strlen(SBOX_INTERP) + 1;
		data->d_buf = strdup(SBOX_INTERP);
		if (!data->d_buf)
			error(strerror(errno));
#else
		memcpy(data->d_buf, "/tmp/ld-fooxx.so.2", strlen((char *) data->d_buf));
#endif

		elf_flagdata(data, ELF_C_SET, ELF_F_DIRTY);

		if (elf_update(elf, ELF_C_NULL) < 0)
			elf_error();
	}

	if (elf_update(elf, ELF_C_WRITE) < 0)
		elf_error();

	elf_end(elf);
	return 0;
}
